﻿using System;
using System.Collections.Generic;

// Base model interface
public interface IBaseModel
{
    int Predict(int input);
}

// Concrete model classes
public class ModelA : IBaseModel
{
    public int Predict(int input)
    {
        // Perform prediction using Model A's algorithm
        return input * 2;
    }
}

public class ModelB : IBaseModel
{
    public int Predict(int input)
    {
        // Perform prediction using Model B's algorithm
        return input + 5;
    }
}

// Ensemble class
public class Ensemble
{
    private List<IBaseModel> models = new List<IBaseModel>();

    public void AddModel(IBaseModel model)
    {
        models.Add(model);
    }

    public void RemoveModel(IBaseModel model)
    {
        // Remove model from the ensemble
        // ...
    }

    public int Predict(int input)
    {
        int sum = 0;
        foreach (IBaseModel model in models)
        {
            int prediction = model.Predict(input);
            sum += prediction;
        }
        return sum / models.Count; // Return average prediction
    }
}

class EnsembleMain
{
    static void Main(string[] args)
    {
        // Create ensemble
        Ensemble ensemble = new Ensemble();

        // Create and add models to the ensemble
        ModelA modelA = new ModelA();
        ModelB modelB = new ModelB();
        ensemble.AddModel(modelA);
        ensemble.AddModel(modelB);

        // Make predictions using the ensemble
        int input = 10;
        int prediction = ensemble.Predict(input);
        Console.WriteLine("Ensemble prediction: " + prediction);

        // Keep the console window open
        Console.ReadLine();
    }
}
